<%	'**************************************************************************
	'* Common code for the weekly pool.                                       *
	'**************************************************************************

	'--------------------------------------------------------------------------
	' Removes weekly player scoring and winner data stored in the database for
	' the week specified.
	'
	' Note: This should be called anytime a change is made to a weekly pool
	' entry or any time a regular season game is updated. I.e., any change to
	' the WeeklyPicks or Schedule tables.
	'--------------------------------------------------------------------------
	sub ClearWeeklyResultsCache(week)

		dim sql

		'Clear any pick and confidence scores saved in the WeeklyScoring table.
		sql = "UPDATE WeeklyScoring SET"_
		    & " PickScore = NULL," _
		    & " ConfidenceScore = NULL" _
			& " WHERE Week = " & week
		call DbConn.Execute(sql)

		'Remove the winners from the WeeklyWinners table.
		sql = "DELETE FROM WeeklyWinners WHERE Week = " & week
		call DbConn.Execute(sql)

	end sub

	'--------------------------------------------------------------------------
	' Returns the number of users who made picks for the specified week.
	'--------------------------------------------------------------------------
	function GetWeeklyEntryCount(week)

		dim sql, rs

		GetWeeklyEntryCount = 0
		sql = "SELECT COUNT(*) As Total FROM WeeklyScoring" _
 		    & " WHERE Week = " & week
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyEntryCount = rs.Fields("Total").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the number of correct picks made by the specified user for the
	' given week. If the user did not enter any picks that week, an empty
	' string is returned.
	'--------------------------------------------------------------------------
	function GetWeeklyPickScore(username, week)

		dim sql, rs, resultField, tieTotal

		GetWeeklyPickScore = ""

		'If the user made no picks for that week, exit.
		if not InWeeklyPool(username, week) then
			exit function
		end if

		'See if we have the score already calculated and saved in the database.
		sql = "SELECT PickScore FROM WeeklyScoring" _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Week = " & week _
			& " AND PickScore IS NOT NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyPickScore = rs.Fields("PickScore").Value
			exit function
		end if

		'Determine which result field to use.
		resultField = "Result"
		if USE_POINT_SPREADS then
			resultField = "ATSResult"
		end if

		'Total the number of correct picks.
		sql = "SELECT COUNT(*) AS Total" _
		    & " FROM WeeklyPicks, Schedule" _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Schedule.Week = " & week _
		    & " AND WeeklyPicks.GameID = Schedule.GameID" _
		    & " AND WeeklyPicks.Pick = Schedule." & resultField _
		    & " AND Schedule." & resultField & " IS NOT NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyPickScore = rs.Fields("Total").Value
		end if

		'If tie picks are not allowed, add a half point for any tied games.
		if not ALLOW_TIE_PICKS then
			sql = "SELECT COUNT(*) AS Total" _
			    & " FROM WeeklyPicks, Schedule" _
			    & " WHERE Username = '" & SqlEscape(username) & "'" _
			    & " AND Schedule.Week = " & week _
			    & " AND WeeklyPicks.GameID = Schedule.GameID" _
			    & " AND WeeklyPicks.Pick <> ''" _
			    & " AND Schedule." & resultField & " = '" & TIE_STR & "'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				tieTotal = rs.Fields("Total").Value
				if IsNumeric(tieTotal) then
					GetWeeklyPickScore = GetWeeklyPickScore + tieTotal / 2
				end if
			end if
		end if

		'To improve performance, save the score in the database.
		sql = "UPDATE WeeklyScoring SET" _
		    & " PickScore = " & GetWeeklyPickScore _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Week = " & week
		call DbConn.Execute(sql)

	end function

	'--------------------------------------------------------------------------
	' Returns the confidence score for picks made by the specified user for the
	' given week. If the user did not enter any picks that week, an empty
	' string is returned.
	'--------------------------------------------------------------------------
	function GetWeeklyConfidenceScore(username, week)

		dim sql, rs, resultField, tieTotal

		GetWeeklyConfidenceScore = ""

		'If the user made no picks for that week, exit.
		if not InWeeklyPool(username, week) then
			exit function
		end if

		'See if we have the score already calculated and saved in the database.
		sql = "SELECT ConfidenceScore FROM WeeklyScoring" _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Week = " & week _
			& " AND ConfidenceScore IS NOT NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyConfidenceScore = rs.Fields("ConfidenceScore").Value
			exit function
		end if

		'Determine which result field to use.
		resultField = "Result"
		if USE_POINT_SPREADS then
			resultField = "ATSResult"
		end if

		'Total the confidence points for each correct pick.
		sql = "SELECT SUM(Confidence) AS Total" _
		    & " FROM WeeklyPicks, Schedule" _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Schedule.Week = " & week _
		    & " AND WeeklyPicks.GameID = Schedule.GameID" _
		    & " AND WeeklyPicks.Pick = Schedule." & resultField _
		    & " AND Schedule." & resultField & " IS NOT NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyConfidenceScore = rs.Fields("Total").Value
			if not IsNumeric(GetWeeklyConfidenceScore) then
				GetWeeklyConfidenceScore = 0
			end if
		end if

		'If tie picks are not allowed, add half points for any tied games.
		if not ALLOW_TIE_PICKS then
			sql = "SELECT SUM(Confidence) AS Total" _
			    & " FROM WeeklyPicks, Schedule" _
			    & " WHERE Username = '" & SqlEscape(username) & "'" _
			    & " AND Schedule.Week = " & week _
			    & " AND WeeklyPicks.GameID = Schedule.GameID" _
			    & " AND Schedule." & resultField & " = '" & TIE_STR & "'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				tieTotal = rs.Fields("Total").Value
				if IsNumeric(tieTotal) then
					GetWeeklyConfidenceScore = GetWeeklyConfidenceScore + tieTotal / 2
				end if
			end if
		end if

		'To improve performance, save the score in the database.
		sql = "UPDATE WeeklyScoring SET" _
		    & " ConfidenceScore = " & GetWeeklyConfidenceScore _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Week = " & week
		call DbConn.Execute(sql)

	end function

	'--------------------------------------------------------------------------
	' Returns an array of users currently in the given week's pool.
	'--------------------------------------------------------------------------
	function GetWeeklyPlayersList(week)

		dim list, sql, rs

		GetWeeklyPlayersList = ""

		sql = "SELECT Username FROM WeeklyScoring" _
		   & " WHERE Week = " & week _
		   & " ORDER BY Username"
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			if not IsArray(list) then
				redim list(0)
			else
				redim preserve list(UBound(list) + 1)
			end if
			list(UBound(list)) = rs.Fields("Username").Value
			rs.MoveNext
		loop
		GetWeeklyPlayersList = list

	end function

	'--------------------------------------------------------------------------
	' Returns the amount in the pot for the given week.
	'--------------------------------------------------------------------------
	function GetWeeklyPotAmount(week)

		GetWeeklyPotAmount = GetWeeklyEntryCount(week) * WEEKLY_ENTRY_FEE
		if ENABLE_OVERALL_WEEKLY then
			GetWeeklyPotAmount = (1 - OVERALL_WEEKLY_SHARE) * GetWeeklyPotAmount
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the actual point total of the last scheduled game of the week
	' specified. If those scores are not available, an empty string is
	' returned.
	'--------------------------------------------------------------------------
	function GetWeeklyTiebreakerActual(week)

		dim sql, rs

		GetWeeklyTiebreakerActual = ""
		sql = "SELECT VisitorScore + HomeScore AS Total" _
	       & " FROM Schedule" _
		   & " WHERE Week = " & week _
		   & " ORDER BY [DateTime] DESC"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			if not IsNumeric(rs.Fields("Total").Value) then
				exit function
			end if
			GetWeeklyTiebreakerActual = rs.Fields("Total").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the specified user's tiebreaker guess for the given week. An
	' empty string is returned if no value is found in the database.
	'--------------------------------------------------------------------------
	function GetWeeklyTiebreakerGuess(username, week)

		dim sql, rs

		GetWeeklyTiebreakerGuess = ""

		sql = "SELECT Tiebreaker" _
		   & " FROM WeeklyScoring" _
		   & " WHERE Username = '" & SqlEscape(username) & "'" _
		   & " AND Week = " & week
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyTiebreakerGuess = rs.Fields("Tiebreaker").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the date and time of the first game scheduled for the given week.
	'--------------------------------------------------------------------------
	function GetWeeklyStartDateTime(n)

		dim sql, rs

		GetWeeklyStartDateTime = Now()
		sql = "SELECT [DateTime]" _
		    & " FROM Schedule" _
		    & " WHERE Week = " & n _
			& " ORDER BY [DateTime]"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetWeeklyStartDateTime = CDate(rs.Fields("DateTime").Value)
		end if

	end function

	'--------------------------------------------------------------------------
	' Determines the winner of the given weekly pool based on picks. An array
	' of those player names is returned. If the pool has not been concluded, an
	' empty string is returned instead.
	'--------------------------------------------------------------------------
	function GetWeeklyWinnersList(week)

		dim sql, rs
		dim tbTarget
		dim highScore, lowDiff
		dim username, score, tb, diff
		dim list

		GetWeeklyWinnersList = ""

		'Exit if not all games have been completed.
		if GetGameCount(week) <> GetCompletedGameCount(week) then
			exit function
		end if

		'See if we have the winners already calculated and stored in the
		'database.
		sql = "SELECT Username FROM WeeklyWinners" _
		    & " WHERE Week = " & week _
			& " ORDER BY Username"
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			if not IsArray(list) then
				redim list(0)
			else
				redim preserve list(UBound(list) + 1)
			end if
			list(UBound(list)) = rs.Fields("Username").Value
			rs.MoveNext
		loop
		if IsArray(list) then
			GetWeeklyWinnersList = list
			exit function
		end if

		'Get point total for the tiebreaker, exit if not available.
		tbTarget = GetWeeklyTiebreakerActual(week)
		if not IsNumeric(tbTarget) then
			exit function
		end if

		'Initialize current high score and tiebreaker difference.
		highScore = -1
		lowDiff   =  0

		'Check each player who has an entry.
		sql = "SELECT * FROM WeeklyScoring" _
		   & " WHERE Week = " & week _
		   & " ORDER BY Username"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			do while not rs.EOF
				username = rs.Fields("Username").Value

				'Find the player's score and tiebreaker.
				if USE_CONFIDENCE_POINTS then
					score = rs.Fields("ConfidenceScore").Value
					if IsNull(score) then
						score = GetWeeklyConfidenceScore(username, week)
					end if
				else
					score = rs.Fields("PickScore").Value
					if IsNull(score) then
						score = GetWeeklyPickScore(username, week)
					end if
				end if
				tb = rs.Fields("Tiebreaker").Value
				if IsNumeric(score) and IsNumeric(tb) then

					'Compare this player's score to the current highest.
					diff = Abs(tbTarget - tb)

					'If this player has a higher score, or the same score and a
					'closer tiebreaker, make the player the winner.
					if score > highScore or (score = highScore and diff < lowDiff) then
						redim list(0)
						list(0)   = username
						highScore = score
						lowDiff   = diff

					'Otherwise, if this player has the same score and same
					'tiebreaker difference, add the player to the winners list.
					elseif score = highScore and diff = lowDiff then
						redim preserve list(UBound(list) + 1)
						list(UBound(list)) = username
					end if
				end if
				rs.MoveNext
			loop
		end if

		'To improve performance, store the winners in the database. A separate
		'table named WeeklyWinners is used for this.
		if IsArray(list) then
			dim i
			for i = LBound(list) to UBound(list)
				sql = "INSERT INTO WeeklyWinners" _
				   & " (Week, Username)" _
				   & " VALUES(" & week & ", '" & SqlEscape(list(i)) & "')"
				call DbConn.Execute(sql)
			next
		end if

		GetWeeklyWinnersList = list

	end function

	'--------------------------------------------------------------------------
    ' Returns an array of all users who have at least one weekly pool entry.
    '--------------------------------------------------------------------------
    function GetAllWeeklyPlayersList()

        dim list, sql, rs

        GetAllWeeklyPlayersList = ""

        sql = "SELECT DISTINCT Username FROM WeeklyScoring ORDER BY Username"
        set rs = DbConn.Execute(sql)
        do while not rs.EOF
            if not IsArray(list) then
                redim list(0)
            else
                redim preserve list(UBound(list) + 1)
            end if
            list(UBound(list)) = rs.Fields("Username").Value
            rs.MoveNext
        loop
        GetAllWeeklyPlayersList = list

    end function

	'--------------------------------------------------------------------------
	' Determines the winner of the overall weekly pool based on picks. An array
	' of those player names is returned. If not all weekly pools have been
	' concluded, an empty string is returned instead.
	'--------------------------------------------------------------------------
	function GetOverallPickWinnersList()

		dim users
		dim sql, rs
		dim score, highScore
		dim i
		dim list

		GetOverallPickWinnersList = ""

		'Make sure the overall pool is enabled all games have been completed.
		if not IsRegularSeasonComplete() then
			exit function
		end if

		'Ensure all weekly scores have been calculated.
		call SetAllWeeklyScores()

		'Get a list of users.
		users = GetAllWeeklyPlayersList()
		if not IsArray(users) then
			exit function
		end if

		'Initialize the current high score.
		highScore = -1

		'Find the best total score among the players.
		for i = 0 to UBound(users)

			'Get the player's total score.
			sql = "SELECT SUM(PickScore) AS Total FROM WeeklyScoring" _
			   & " WHERE Username = '" & SqlEscape(users(i)) & "'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				score = rs.Fields("Total").Value

				'If this player has a higher score, make the player the winner.
				if score > highScore then
					redim list(0)
					list(0)   = users(i)
					highScore = score

				'Otherwise, if this player has the same score, add the player
				'to the winners list.
				elseif score = highScore then
					redim preserve list(UBound(list) + 1)
					list(UBound(list)) = users(i)
				end if
			end if
		next

		if IsArray(list) then
			GetOverallPickWinnersList = list
		end if

	end function

	'--------------------------------------------------------------------------
	' Determines the winner of the overall weekly pool based on confidence
	' points. An array of those player names is returned. If not all weekly
	' pools have been concluded, an empty string is returned instead.
	'--------------------------------------------------------------------------
	function GetOverallConfidenceWinnersList()

		dim users
		dim sql, rs
		dim score, highScore
		dim i
		dim list

		GetOverallConfidenceWinnersList = ""

		'Make sure the overall pool is enabled all games have been completed.
		if not IsRegularSeasonComplete() then
			exit function
		end if

		'Ensure all weekly scores have been calculated.
		call SetAllWeeklyScores()

		'Get a list of users.
		users = GetAllWeeklyPlayersList()
		if not IsArray(users) then
			exit function
		end if

		'Initialize the current high score.
		highScore = -1

		'Find the best total score among the players.
		for i = 0 to UBound(users)

			'Get the player's total score.
			sql = "SELECT SUM(ConfidenceScore) AS Total FROM WeeklyScoring" _
			   & " WHERE Username = '" & SqlEscape(users(i)) & "'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				score = rs.Fields("Total").Value

				'If this player has a higher score, make the player the winner.
				if score > highScore then
					redim list(0)
					list(0)   = users(i)
					highScore = score

				'Otherwise, if this player has the same score, add the player
				'to the winners list.
				elseif score = highScore then
					redim preserve list(UBound(list) + 1)
					list(UBound(list)) = users(i)
				end if
			end if
		next

		if IsArray(list) then
			GetOverallConfidenceWinnersList = list
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the current amount in the pot for the overall weekly pool.
	'--------------------------------------------------------------------------
	function GetOverallPotAmount()

		dim week
		dim sql, rs

		GetOverallPotAmount = 0

		'Find the total number of entries so far.
		week = GetCurrentWeek()
		sql = "SELECT COUNT(*) AS Total FROM WeeklyScoring WHERE Week <=" & week
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetOverallPotAmount = OVERALL_WEEKLY_SHARE * WEEKLY_ENTRY_FEE * rs.Fields("Total").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns true if the given user has an entry for the specified week.
	'--------------------------------------------------------------------------
	function InWeeklyPool(username, week)

		dim sql, rs

		InWeeklyPool = false
		if GetWeeklyTiebreakerGuess(username, week) <> "" then
			InWeeklyPool = true
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns true if the all games for the regular season have been completed.
	'--------------------------------------------------------------------------
	function IsRegularSeasonComplete()

		dim sql, rs

		IsRegularSeasonComplete = false

		sql = "SELECT COUNT(*) AS Total FROM Schedule WHERE Result IS NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			if rs.Fields("Total").Value = 0 then
				IsRegularSeasonComplete = true
			end if
		end if

	end function

	'--------------------------------------------------------------------------
	' This subroutine will ensure that all weekly player scores are calculated
	' and cached in the WeeklyScoring table.
	'--------------------------------------------------------------------------
	sub SetAllWeeklyScores()

		dim sql, rs
		dim score

		sql = "SELECT Username, Week FROM WeeklyScoring WHERE PickScore IS NULL"
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			score = GetWeeklyPickScore(rs.Fields("Username").Value, rs.Fields("Week").Value)
			if USE_CONFIDENCE_POINTS then
				score = GetWeeklyConfidenceScore(rs.Fields("Username").Value, rs.Fields("Week").Value)
			end if
			rs.MoveNext
		loop

	end sub %>
