//*****************************************************************************
// Confidence points status display code.
//*****************************************************************************

//=============================================================================
// Module: confPtsStatus
//
// Used to update the confidence points status display on an entry form when
// changes are made to the point assignments.
//
// Example usage:
//
//   // Update the confidence points status display.
//   confPtsStatus.update(formEl);
//
// and:
//
//  // Initialize the confidence points status display on page load.
//  domUtils.onready(function () {
//    confPtsStatus.update(document.getElementById("entryForm"));
//  });
//
// Note that the form should contain two hidden input fields which specify the
// number of games and the ID of the table cell that contains the status
// display:
//
//  <input type="hidden" name="confPtsNumGames" value="16" />
//  <input type="hidden" name="confPtsStatusID" value="confPtsStatus" />
//=============================================================================
var confPtsStatus = function () {

	//=========================================================================
	// Public properties and methods.
	//=========================================================================
	return {

		//---------------------------------------------------------------------
		// Updates the confidence points status display.
		//---------------------------------------------------------------------
		update : function (formEl) {

			// If the form does not exist, exit.
			if (formEl == null)
				return false;

			// If not already done, find the number of games and the table
			// cell containing the status display.
			if (formEl.numGames == null) {
				try {

					// Get these values from special input fields in the form.
					formEl.numGames = formEl.elements["confPtsNumGames"].value;
					formEl.statusEl = document.getElementById(formEl.elements["confPtsStatusID"].value);

					// Make the parent row of the status display visible.
					formEl.statusEl.parentNode.style.display = "";
				}
				catch (ex) {}
			}

			// Exit if neither can be found.
			if (formEl.numGames == null || formEl.statusEl == null)
				return false;

			// Build a array of booleans for each point value.
			var pointsList = [];

			// Set the flags.
			var inputEl, pts;
			for (var i = 0; i < formEl.numGames; i++)
				pointsList[i] = 0;
			for (i = 1; i <= formEl.numGames; i++) {
				if ((inputEl = formEl.elements["conf-" + i]) == null)
					inputEl = formEl.elements["lockedConf-" + i];
				if (inputEl != null) {
					pts = parseFloat(inputEl.value);
					if (!isNaN(pts) && pts == parseInt(pts) && pts >= 1 && pts <= formEl.numGames)
						pointsList[pts - 1] += 1;
				}
			}

			// Build the display.
			var spanEl;
			while (formEl.statusEl.firstChild != null)
				formEl.statusEl.removeChild(formEl.statusEl.firstChild);
			formEl.statusEl.appendChild(document.createTextNode("Confidence Points:"));
			for (i = 1; i <= formEl.numGames; i++) {
				spanEl = document.createElement("SPAN");
				spanEl.appendChild(document.createTextNode(i));
				if (pointsList[i - 1] == 0)
					spanEl.className = "availPts";
				if (pointsList[i - 1] == 1)
					spanEl.className = "usedPts";
				if (pointsList[i - 1] > 1)
					spanEl.className = "duplPts";
				formEl.statusEl.appendChild(document.createTextNode(" "));
				formEl.statusEl.appendChild(spanEl);
			}
			return false;
		}
	}
}();

