//*****************************************************************************
// Date and time picker code.
//*****************************************************************************

//=============================================================================
// Module: datePicker
//
// Handles the pop-up date selector for form fields.
//
// Example usage:
//
//   // Show the pop-up calendar date selection display.
//   datePicker.open(formEl, dayFieldName, dateFieldName);
//
// Note that two form fields are used: one for showing the day of the week
// the selected date falls on ("Mon", "Tue", etc.) and the other for displaying
// the selected date in 'm/d/yyyy' format.
//=============================================================================
var datePicker = function () {

	// Date picker display element.
	var calendarEl = null;

	// Target form fields.
	var targetDayField  = null;
	var targetDateField = null;

	// The currently selected date.
	var selectedDate = null;

	// Month and day names.
	var monthNames = [ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ];
	var dayNames   = [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ];

	//-------------------------------------------------------------------------
	// Returns the name of the month represented by the given date.
	//-------------------------------------------------------------------------
	function getMonthName(date) {

		return monthNames[date.getMonth()];
	}

	//-------------------------------------------------------------------------
	// Returns the name of the day represented by the given date.
	//-------------------------------------------------------------------------
	function getDayName(date) {

		return dayNames[date.getDay()];
	}

	//-------------------------------------------------------------------------
	// Returns the number of days in the month represented by the given date.
	//-------------------------------------------------------------------------
	function getDays(date) {

		tmpDate = new Date(Date.parse(date));
		var m = tmpDate.getMonth();
		var d = 28;
		do {
			d++;
			tmpDate.setDate(d);
		} while (tmpDate.getMonth() == m);

		return d - 1;
	}

	//-------------------------------------------------------------------------
	// Sets the calendar display to reflect the currently selected date.
	//-------------------------------------------------------------------------
	function setCalendar() {

		var el, tbodyEl, rowEl, cellEl, linkEl;
		var tmpDate, tmpDate2;
		var i, j;

		// Update the month/year in the header.
		var el = document.getElementById("calendarMonth").firstChild;
		el.nodeValue = getMonthName(selectedDate) + "\u00a0" + selectedDate.getFullYear();

		// Start with the first day of the month and go back as necessary to
		// the previous Sunday.
		tmpDate = new Date(Date.parse(selectedDate));
		tmpDate.setDate(1);
		while (tmpDate.getDay() != 0)
			tmpDate.setDate(tmpDate.getDate() - 1)

		// Go through each calendar day cell in the table body and update it.
		tbodyEl = document.getElementById("calendarBody");
		for (i = 0; i < tbodyEl.rows.length; i++) {
			rowEl = tbodyEl.rows[i];

			// Loop through a week.
			for (j = 0; j < rowEl.cells.length; j++) {
				cellEl = rowEl.cells[j];
				linkEl = cellEl.firstChild;

				// Set the date display.
				linkEl.date = new Date(Date.parse(tmpDate));
				linkEl.title = "Select " + getDayName(tmpDate) + ", " + getMonthName(tmpDate) + " " + tmpDate.getDate() + ", " + tmpDate.getFullYear() + ".";
				linkEl.firstChild.nodeValue = tmpDate.getDate();

				// Set style for dates outside the target month.
				if (tmpDate.getMonth() != selectedDate.getMonth())
					domUtils.addClass(linkEl, "otherMonth");
				else
					domUtils.removeClass(linkEl, "otherMonth");

				// Highlight the selected date.
				if (Date.parse(tmpDate) == Date.parse(selectedDate))
					domUtils.addClass(cellEl, "selected");
				else
					domUtils.removeClass(cellEl, "selected");

				// Go to the next day.
				tmpDate.setDate(tmpDate.getDate() + 1);
			}
		}
	}

	//=========================================================================
	// Public properties and methods.
	//=========================================================================
	return {

		//---------------------------------------------------------------------
		// Displays the date picker using the given form fields.
		//---------------------------------------------------------------------
		open : function (formEl, dayFieldName, dateFieldName) {

			// If we don't have the date picker display element yet, find it.
			if (calendarEl == null)
				calendarEl = document.getElementById("calendar");

			// If it cannot be found, exit.
			if (calendarEl == null)
				return false;

			// Make sure the date picker and time picker are both closed.
			this.close();
			timePicker.close();

			// Set the target form fields and date.
			targetDayField  = formEl.elements[dayFieldName];
			targetDateField = formEl.elements[dateFieldName];
			selectedDate = new Date(targetDateField.value);
			if (isNaN(selectedDate.valueOf()))
				selectedDate = new Date();

			// Position and show the date picker.
			var pt = domUtils.offset(targetDayField);
			pt.x += 8;
			pt.y += targetDayField.offsetHeight - 4;
			calendarEl.style.left = pt.x + "px";
			calendarEl.style.top  = pt.y + "px";
			calendarEl.style.display = "block";
			setCalendar();

			// Highlight the target form fields.
			domUtils.addClass(targetDayField,  "activeDateTime");
			domUtils.addClass(targetDateField, "activeDateTime");

			return false;
		},

		//---------------------------------------------------------------------
		// Hides the date picker.
		//---------------------------------------------------------------------
		close : function () {

			// Exit if the date picker display has not been set yet.
			if (calendarEl == null)
				return false;

			// Hide the date picker display.
			calendarEl.style.display = "";

			// Remove highlighting form the target form fields.
			if (targetDayField != null)
				domUtils.removeClass(targetDayField, "activeDateTime");
			if (targetDateField != null)
				domUtils.removeClass(targetDateField, "activeDateTime");
			targetDayField  = null;
			targetDateField = null;

			return false;
		},

		//---------------------------------------------------------------------
		// Event handlers for previous/next month links on the date picker
		// display.
		//---------------------------------------------------------------------
		monthClick : function (n) {

			// Save the day of month if not already set.
			if (selectedDate.savedDate == null)
				selectedDate.savedDate = selectedDate.getDate();

			// Set the day to the first of the month (to avoid rolling).
			selectedDate.setDate(1);

			// Add the specified number of months.
			selectedDate.setMonth(selectedDate.getMonth() + n);

			// Restore the saved day of month, if possible.
			selectedDate.setDate(Math.min(selectedDate.savedDate, getDays(selectedDate)));

			// Update the display.
			setCalendar();

			return false;
		},

		//---------------------------------------------------------------------
		// Event handler for date links on the calendar display.
		//---------------------------------------------------------------------
		dateClick : function (link) {

			// Change the selected date and update the calendar.
			if (link.date != null) {
				selectedDate = new Date(Date.parse(link.date));
				setCalendar();
			}

			return false;
		},

		//---------------------------------------------------------------------
		// Event handler for the 'Ok' button on the date picker display.
		//---------------------------------------------------------------------
		okClick : function () {

			// Format the selected day as "ddd".
			var s = getDayName(selectedDate).substr(0, 3);

			// Format the selected date as "m/d/yyyy".
			var t = String(selectedDate.getMonth() + 1)
				+ "/"
				+ String(selectedDate.getDate())
				+ "/"
				+ String(selectedDate.getFullYear());

			// Set the target day and date fields.
			targetDayField.value  = s;
			targetDateField.value = t;

			// Close the calendar.
			this.close();

			return false;
		},

		//---------------------------------------------------------------------
		// Event handler for the 'Close' button on the date picker display.
		//---------------------------------------------------------------------
		cancelClick : function () {

			// Hide the date picker display.
			this.close();

			return false;
		}
	}
}();

//=============================================================================
// Module: timePicker
//
// Handles the pop-up time selector for form fields.
//
// Example usage:
//
//   // Show the pop-up time selection display.
//   timePicker.open(formEl, timeFieldName);
//=============================================================================
var timePicker = function () {

	// Time picker display elements.
	var clockEl     = null;
	var clockFormEl = null;

	// Target form field.
	var targetTimeField = null;

	//-------------------------------------------------------------------------
	// Sets one part of a time value on the time picker display.
	//-------------------------------------------------------------------------
	function setClockField(name, value) {

		// Select the specified value in the named drop-down list.
		var selectEl = clockFormEl.elements[name];
		for (var i = 0; i < selectEl.options.length; i++)
			if (selectEl.options[i].value == value)
				selectEl.options[i].selected = true;
			else
				selectEl.options[i].selected = false;
	}

	//=========================================================================
	// Public properties and methods.
	//=========================================================================
	return {

		//---------------------------------------------------------------------
		// Displays the time picker using the given form field.
		//---------------------------------------------------------------------
		open : function (formEl, timeFieldName) {

			// If we don't have the time picker display element yet, find it.
			if (clockEl == null)
				clockEl = document.getElementById("clock");

			// If it cannot be found, exit.
			if (clockEl == null)
				return false;

			// Get the time picker form element.
			if (clockFormEl == null)
				clockFormEl = document.getElementById("clockForm");

			// Make sure the date picker and time picker are both closed.
			datePicker.close();
			this.close();

			// Set the target form field.
			targetTimeField = formEl.elements[timeFieldName];

			// Position and show the time picker display.
			var pt = domUtils.offset(targetTimeField);
			pt.x += 8;
			pt.y += targetTimeField.offsetHeight - 4;
			clockEl.style.left = pt.x + "px";
			clockEl.style.top  = pt.y + "px";
			clockEl.style.display = "block";

			// Get the time in the target form field.
			var list = targetTimeField.value.split(":");
			var hour, minute, meridiem;
			if (list.length >= 2) {
				hour = list[0];
				minute = Math.round(list[1] / 5) * 5;
				list = targetTimeField.value.split(" ");
				meridiem = list[1].toLowerCase();
			}
			else {

				// Use the current time.
				var timeNow = new Date()
				hour = (timeNow.getHours()) % 12;
				minute = Math.round(timeNow.getMinutes() / 5) * 5;
				meridiem = timeNow.getHours() >= 12 ? "pm" : "am";
			}

			// Set the time in the display.
			setClockField("clockHour", hour);
			setClockField("clockMinute", minute);
			setClockField("clockMeridiem", meridiem);

			// Highlight the target form field.
			domUtils.addClass(targetTimeField, "activeDateTime");

			return false;
		},
	
		//---------------------------------------------------------------------
		// Hides the time picker.
		//---------------------------------------------------------------------
		close : function () {

			// Exit if the time picker display has not been set yet.
			if (clockEl == null)
				return false;

			// Hide the time picker display.
			clockEl.style.display = "";

			// Remove highlighting form the target form field.
			if (targetTimeField != null)
				domUtils.removeClass(targetTimeField, "activeDateTime");
			targetTimeField = null;

			return false;
		},

		//---------------------------------------------------------------------
		// Event handler for the 'Ok' button on the time picker display.
		//---------------------------------------------------------------------
		okClick : function () {

			// Format the selected time as "h:mm:ss tt".
			var s = clockFormEl.elements["clockHour"].value
				+ ":" + clockFormEl.elements["clockMinute"].value
				+ ":00 " + clockFormEl.elements["clockMeridiem"].value;

			// Set the target form field.
			targetTimeField.value = s;
			
			// Hide the time picker display.
			this.close();

			return false;
		},

		//---------------------------------------------------------------------
		// Event handler for the 'Close' button on the time picker display.
		//---------------------------------------------------------------------
		cancelClick : function () {

			// Hide the time picker display.
			this.close();

			return false;
		}
	}
}();

