//*****************************************************************************
// Table sort code.
//*****************************************************************************

//=============================================================================
// Module: tableSort
//
// Used for sorting tables based on a set of parameters.
//
// Example usage:
//
//   // Define a set of sort parameters.
//   var mySortCommand =
//   {
//     initial     : false,
//     tbodyID     : "myTableBody",
//     theadID     : "myTableHead",
//     sortColumns : [
//       { column : -1, descending : false }, // Primary key.
//       { column : -2, descending : true },  // Secondary key (reverse order).
//       { column :  0, descending : false }  // Tertiary key.
//     ],
//     highlightColumns : {
//       bodyColumns : [-2, -1], // Highlight the last two body columns.
//       headColumns : [-1]      // Highlight the last head column.
//     }
//   }
//
//   // Run the sort command.
//   tableSort.sort(mySortCommand);
//
// The above example will sort the table using the last column as the primary
// sort key, the next to last column as the secondary sort key (in descending
// order) and the first column as the tertiary sort key. It will also highlight
// the last two columns in the table body and the last column in the table
// head (useful, for example, if the column header spans two columns).
//
// Calling a sort command again will toggle the sort order (provided no other
// sort command has been used on that given table).
//
// Details for the parameters are shown below.
//
// SortCommand object format:
//
//  initial          : Set to true if this sort matches the initial sort order
//                     of the table.
//  bodyID           : ID of the table body element.
//  headID           : ID of the table head element.
//  sortColumns      : Array of SortColumn objects (primary, secondary, etc.).
//	highlightColumns : A SortHighlight object.
//
// SortColumn object format:
//
//  column     : index* of the column to be sorted.
//  descending : descending sort flag.
//
// SortHighlight object format:
//
// bodyColumns : Array of column indices* to be highlighted in the body.
// headColumns : Array of column indices* to be highlighted in the head (only
//               applied to the first row in the table head).
//
// *Use negative values to specify columns from the right. E.g., -1 gives the
// the last column in a row, -2 gives the next to last column, etc.
//=============================================================================
var tableSort = function () {

	// Regular expressions for special data formats.
	var currencyRegex   = new RegExp("^(-?)\\$(\\d+.\\d{2})$");
	var differenceRegex = new RegExp("^\\((\\d+)\\)$");
	var percentRegex    = new RegExp("^\\((\\d+.?\\d*)\\)$");

	// Regular expressions for normalizing white space.
	var ldTrSpRegex = new RegExp("^\\s*|\\s*$", "g");
	var multSpRegex = new RegExp("\\s\\s+", "g");

	//-------------------------------------------------------------------------
	// Returns the text within a table cell.
	//-------------------------------------------------------------------------
	function getCellText(cellEl) {

		// See if we've extracted the text already.
		if (cellEl.tableSortValue != null)
			return cellEl.tableSortValue;

		// Find and concatenate the values of all text nodes contained within
		// the element.
		var s = "";
		for (var i = 0; i < cellEl.childNodes.length; i++) {
			if (cellEl.childNodes[i].nodeType == document.TEXT_NODE) {

				// Get the text, replacing any no-breaking spaces.
				s += cellEl.childNodes[i].nodeValue.replace(/\u00a0/, " ");
			}
			else {
				if (cellEl.childNodes[i].nodeType == document.ELEMENT_NODE && cellEl.childNodes[i].tagName == "BR")
					s += " ";
				else {

					// Use recursion to get text within sub-elements.
					s += getCellText(cellEl.childNodes[i]);
				}
			}
		}

		// Normalize any white space in the string.
		s = s.replace(multSpRegex, " ");  // Collapse any multiple white space.
		s = s.replace(ldTrSpRegex, "");   // Remove and leading and trailing white space.

		// For efficiency, save the text string.
		cellEl.tableSortValue = s;

		return s;
	}

	//-------------------------------------------------------------------------
	// Compares two text values based on their format.
	//-------------------------------------------------------------------------
	function compareValues(v1, v2) {

		// Convert text to lowercase.
		v1 = v1.toLowerCase();
		v2 = v2.toLowerCase();

		// Replace any 1/2 character with a decimal string.
		v1 = v1.replace(/\u00bd/, ".5");
		v2 = v2.replace(/\u00bd/, ".5");

		// If the values are in currency format (after removing commas),
		// convert them to numeric strings.
		var t1 = v1.replace(",", "");
		var t2 = v2.replace(",", "");
		if (currencyRegex.test(t1) && currencyRegex.test(t2)) {
			v1 = t1.replace(currencyRegex, "$1$2");
			v2 = t2.replace(currencyRegex, "$1$2");
		}

		// If the values are in the format used for the tie breaker difference,
		// convert them to numeric strings.
		if (differenceRegex.test(v1) && differenceRegex.test(v2)) {
			v1 = v1.replace(differenceRegex, "$1");
			v2 = v2.replace(differenceRegex, "$1");
		}

		// If the values are in percentage format, convert them to numeric
		// strings.
		if (percentRegex.test(v1) && percentRegex.test(v2)) {
			v1 = v1.replace(percentRegex, "$1");
			v2 = v2.replace(percentRegex, "$1");
		}

		// Always rank a numeric value higher than a non-numeric value.
		var f1 = parseFloat(v1);
		var f2 = parseFloat(v2);
		if (!isNaN(f1) && isNaN(f2))
			return 1;
		if (isNaN(f1) && !isNaN(f2))
			return -1;

		// If the values are numeric, convert them to floats.
		f1 = parseFloat(v1);
		f2 = parseFloat(v2);
		if (!isNaN(f1) && !isNaN(f2)) {
			v1 = f1;
			v2 = f2;
		}

		// Compare the two values and return the result.
		if (v1 == v2)
			return 0;
		if (v1 > v2)
			return 1
		return -1;
	}

	//-------------------------------------------------------------------------
	// Resets styles and adds highlighting to a table after a sort.
	//-------------------------------------------------------------------------
	function restyleTable(sortCommand) {

		// Get the table body and head.
		var tbodyEl = document.getElementById(sortCommand.tbodyID);
		var theadEl = document.getElementById(sortCommand.theadID);

		var i, j;
		var rowEl;
		var col;

		// Restyle the table body.
		for (i = 0; i < tbodyEl.rows.length; i++) {
			rowEl = tbodyEl.rows[i];

			// Zebra stripe the rows.
			if (i % 2 != 0)
				domUtils.addClass(rowEl, "alt");
			else
				domUtils.removeClass(rowEl, "alt");

			// Remove any highlighting on cells from a previous sort.
			for (j = 0; j < rowEl.cells.length; j++)
				domUtils.removeClass(rowEl.cells[j], "sortedCol");

			// Now highlight the specifed sort columns.
			for (j = 0; j < sortCommand.highlightColumns.bodyColumns.length; j++) {
				col = sortCommand.highlightColumns.bodyColumns[j];
				if (col < 0)
					col = rowEl.cells.length + col;
				domUtils.addClass(rowEl.cells[col], "sortedCol");
			}
		}

		// Set style classes for the header columns.
		rowEl = theadEl.rows[0];

		// Remove any highlighting on cells from a previous sort.
		for (j = 0; j < rowEl.cells.length; j++)
			domUtils.removeClass(rowEl.cells[j], "sortedCol");

		// Now highlight the specifed header cells.
		for (j = 0; j < sortCommand.highlightColumns.headColumns.length; j++) {
			col = sortCommand.highlightColumns.headColumns[j];
			if (col < 0)
				col = rowEl.cells.length + col;
			domUtils.addClass(rowEl.cells[col], "sortedCol");
		}
	}

	//=========================================================================
	// Public properties and methods.
	//=========================================================================
	return {

		//---------------------------------------------------------------------
		// Sorts a table using the given sort command.
		//---------------------------------------------------------------------
		sort : function (sortCommand) {

			// Get the table body.
			var tbodyEl = document.getElementById(sortCommand.tbodyID);

			// If the first row has a special 'noSort' class name, exit. (This
			// class name is used to indicate that no valid data appears in the
			// table.)
			if (tbodyEl.rows.length > 0 && domUtils.hasClass(tbodyEl.rows[0], "noSort"))
				return false;

			// Do the sort.
			var i, j, k;
			var col;
			var minIndex;
			var v1, v2;
			var cmp;
			var tmpEl;

			// Handle the special case of the first sort command called on the
			// table matching the initial sort order (i.e., if it is the first
			// sort command called on the table then reverse the order).
			if (tbodyEl.lastSortCommand == null && sortCommand.initialSort)
				tbodyEl.lastSortCommand = sortCommand;

		 	// If this sort command was the last one executed for this table,
			// toggle its direction.
			if (tbodyEl.lastSortCommand == sortCommand)
				for (i = 0; i < sortCommand.sortColumns.length; i++)
					sortCommand.sortColumns[i].descending = !sortCommand.sortColumns[i].descending;

			// Do the sort.
			for (i = 0; i < tbodyEl.rows.length - 1; i++) {

				// Start by assuming the current row has the smallest value.
				minIndex = i;

				// Search the remaining rows for a smaller value.
				for (j = i + 1; j < tbodyEl.rows.length; j++) {

					// Compare values in each sort column until a difference is
					// found.
					k = 0;
					do {

						// Get the column index.
						col = sortCommand.sortColumns[k].column;
						if (col < 0)
							col = tbodyEl.rows[minIndex].cells.length + col;

						// Compare the values, negate if the descending sort flag is set.
						try {
							v1 = getCellText(tbodyEl.rows[minIndex].cells[col]);
							v2 = getCellText(tbodyEl.rows[j].cells[col]);
						}
						catch (ex) {
							return false;
						}
						cmp = compareValues(v1, v2);
						if (sortCommand.sortColumns[k].descending)
							cmp = -cmp;

						k++;
					} while (cmp == 0 && k < sortCommand.sortColumns.length);

					// If this row has a smaller value than the current
					// minimum, make it the new minimum.
					if (cmp > 0)
						minIndex = j;
				}

				// By now, we have the remaining row with the smallest value.
				// Remove it from the table and reinsert it before the current
				// row.
				if (minIndex > i)
				{
					tmpEl = tbodyEl.removeChild(tbodyEl.rows[minIndex]);
					tbodyEl.insertBefore(tmpEl, tbodyEl.rows[i]);
				}
			}

			// Restore zebra striping and highlight the sorted column(s).
			restyleTable(sortCommand);

			// Remember this sort command as the last one executed for this
			// table.
			tbodyEl.lastSortCommand = sortCommand;

			return false;
		}
	}
}();

