<%	'**************************************************************************
	'* Common code for the survivor/margin side pool.                         *
	'**************************************************************************

	'--------------------------------------------------------------------------
	' Removes weekly margin pool scores stored in the database for the week
	' specified.
	'
	' Note: This should be called anytime a regular season game is updated.
	' I.e., any change to the Schedule table.
	'--------------------------------------------------------------------------
	sub ClearMarginResultsCache(week)

		dim sql

		sql = "UPDATE SidePicks" _
		   & " SET MarginScore = NULL" _
		   & " WHERE Week >= " & week
		call DbConn.Execute(sql)

	end sub

	'--------------------------------------------------------------------------
	' Removes weekly survivor pool status information stored in the database
	' for the week specified and beyond.
	'
	' Note: This should be called anytime a change is made to a survivor/margin
	' pool entry or any time a regular season game is updated. I.e., any change
	' the SidePicks or Schedule tables.
	'--------------------------------------------------------------------------
	sub ClearSurvivorStatus(week)

		dim sql

		sql = "DELETE FROM SurvivorStatus WHERE Week >= " & week
		call DbConn.Execute(sql)

		'Fix buybacks, if appropriate.
		if SURVIVOR_BUYBACK_LIMIT > 0 then
			call ShiftSurvivorBuybacks()
		end if

	end sub

	'--------------------------------------------------------------------------
	' Returns the number of players who have entries in the survivor/margin
	' pool.
	'--------------------------------------------------------------------------
	function GetSideEntryCount()

		dim sql, rs
      
		GetSideEntryCount = 0
'		sql = "SELECT COUNT(*) AS Total FROM (SELECT DISTINCT Username FROM SidePicks)"
		sql = "SELECT COUNT(DISTINCT Username) AS Total FROM SidePicks"
        set rs = DbConn.Execute(sql)
      	if not (rs.EOF and rs.BOF) then
			GetSideEntryCount = rs.Fields("Total").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the given users survivor/margin pick for the given week. If the
	' user has not made a pick, an empty string is returned.
	'--------------------------------------------------------------------------
	function GetSidePick(username, week)

		dim sql, rs

		GetSidePick = ""
		sql = "SELECT Pick FROM SidePicks" _
		   & " WHERE Username = '" & SqlEscape(username) & "'" _
		   & " AND Week = " & week
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetSidePick = rs.Fields("Pick").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns an array of picks that have already used by the given player.
	'--------------------------------------------------------------------------
	function GetUsedSidePicksList(username)

		dim list, sql, rs

		GetUsedSidePicksList = ""

		sql = "SELECT Pick FROM SidePicks" _
		   & " WHERE Username = '" & SqlEscape(username) & "'"
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			if not IsArray(list) then
				redim list(0)
			else
				redim preserve list(UBound(list) + 1)
			end if
			list(UBound(list)) = rs.Fields("Pick").Value
			rs.MoveNext
		loop
		GetUsedSidePicksList = list

	end function

	'--------------------------------------------------------------------------
	' Returns an array of users currently in the survivor/margin pool.
	'--------------------------------------------------------------------------
	function GetSidePlayersList()

		dim list, sql, rs

		GetSidePlayersList = ""

		sql = "SELECT DISTINCT Username FROM SidePicks" _
		   & " ORDER BY Username"
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			if not IsArray(list) then
				redim list(0)
			else
				redim preserve list(UBound(list) + 1)
			end if
			list(UBound(list)) = rs.Fields("Username").Value
			rs.MoveNext
		loop
		GetSidePlayersList = list
 
	end function

	'--------------------------------------------------------------------------
	' Returns true if the given user has entered any picks in the
	' survivor/margin pool.
	'--------------------------------------------------------------------------
	function InSidePool(username)

		dim sql, rs

		InSidePool = false
		sql = "SELECT COUNT(*) AS Total FROM SidePicks" _
		   & " WHERE Username = '" & SqlEscape(username) & "'"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			if rs.Fields("Total").Value > 0 then
				InSidePool = true
			end if
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns true if the game for the given survivor/margin pick has been
	' locked.
	'--------------------------------------------------------------------------
	function IsSidePickLocked(pick, week)

		dim sql, rs, gameDate

		IsSidePickLocked = false
		if IsWeekLocked(week) then
			IsSidePickLocked = true
			exit function
		end if

		sql = "SELECT [DateTime] FROM Schedule" _
		   & " WHERE Week = " & week _
		   & " AND (VisitorID = '" & pick & "' OR HomeID = '" & pick & "')"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			gameDate = CDate(rs.Fields("DateTime").Value)
			if IsDateTimePast(gameDate) then
				IsSidePickLocked = true
			end if
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the current amount in the pot for the margin pool.
	'--------------------------------------------------------------------------
	function GetMarginPotAmount()

		GetMarginPotAmount = 0

		'Get the base amount.
		GetMarginPotAmount =  GetSideEntryCount() * SIDE_ENTRY_FEE
		if ENABLE_SURVIVOR_POOL then
			GetMarginPotAmount = (1 - SIDE_SURVIVOR_SHARE) * GetMarginPotAmount
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the given user's margin score for the given week. Returns an
	' empty string if no score is available.
	'--------------------------------------------------------------------------
	function GetMarginScore(username, week)

		dim sql, rs
		dim pick, margin

		GetMarginScore = ""

		'Exit if the week is invalid.
		if week < SIDE_START_WEEK then
			exit function
		end if

		'Exit if the player is not in the pool.
		if not InSidePool(username) then
			exit function
		end if

		'See if we have the score already calculated and saved in the database.
		sql = "SELECT MarginScore FROM SidePicks" _
		    & " WHERE Username = '" & SqlEscape(username) & "'" _
		    & " AND Week = " & week _
			& " AND MarginScore IS NOT NULL"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			GetMarginScore = rs.Fields("MarginScore").Value
			exit function
		end if

		'Get the user's pick.
		pick = GetSidePick(username, week)
		if pick <> "" then

			'Find the margin of victory/loss.
			sql = "SELECT VisitorID, VisitorScore, HomeID, HomeScore, Result" _
			   & " FROM Schedule" _
			   & " WHERE Schedule.Week = " & week _
			   & " AND (VisitorID = '" & pick & "' OR HomeID = '" & pick & "')" _
			   & " AND Result IS NOT NULL"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				margin = Abs(rs.Fields("VisitorScore").Value - rs.Fields("HomeScore").Value)
				if rs.Fields("Result").Value = pick then
					GetMarginScore = margin
				else
					if MARGIN_DEDUCT_LOSS then
						GetMarginScore = -margin
					else
						GetMarginScore = 0
					end if
				end if

				'To improve performance, store the score in the database.
				sql = "UPDATE SidePicks SET" _
				    & " MarginScore = " & GetMarginScore _
				    & " WHERE Username = '" & SqlEscape(username) & "'" _
				    & " AND Week = " & week
				call DbConn.Execute(sql)
			end if

		'The user does not have a pick for that week so assign a score if
		'possible.
		else

			'If misses are not deducted and all games are locked, assign a
			'score of zero.
			if not MARGIN_DEDUCT_LOSS then
				if IsWeekLocked(week) then
					GetMarginScore = 0
				end if

			'Otherwise, if all the week's games have been completed, find
			'the greatest score margin and assign that as a negative score.
			elseif GetCompletedGameCount(week) = GetGameCount(week) then
					sql = "SELECT MAX(ABS(VisitorScore - HomeScore)) AS Margin FROM Schedule" _
					   & " WHERE Week = " & week _
					   & " AND Result IS NOT NULL"
					set rs = DbConn.Execute(sql)
					if not (rs.EOF and rs.BOF) then
						margin = rs.Fields("Margin").Value
						GetMarginScore = -margin
					end if
			end if

			if IsNumeric(GetMarginScore) then

				'To improve performance, store the score in the database (with
				'an empty pick).
				sql = "INSERT INTO SidePicks" _
				   & " (Week, Username, Pick, MarginScore)" _
				   & " VALUES(" & week & ", " _
				   & "'" & SqlEscape(username) & "', " _
				   & "'', " _
				   & GetMarginScore & ")"
				call DbConn.Execute(sql)
			end if

		end if

	end function

	'--------------------------------------------------------------------------
	' Determines the winner of the margin pool. An array of those player
	' names is returned. If the pool has not been concluded, or if no players
	' participated in it, an empty string is returned instead.
	'--------------------------------------------------------------------------
	function GetMarginWinnersList()

		dim finalWeek
		dim users, i, week
		dim highScore, mostCorrect
		dim score, correct, n
		dim sql, rs, list

		GetMarginWinnersList = ""

		'If the final week of the regular season has not started, exit.
		finalWeek = GetWeekCount()
		if not IsWeekLocked(finalWeek) then
			exit function
		end if

		'If there were no players participating in the pool, exit.
		users = GetSidePlayersList()
		if not IsArray(users) then
			exit function
		end if

		'If any player does not yet have a margin score for the final week,
		'exit.
		for i = 0 to UBound(users)
			if GetMarginScore(users(i), finalWeek) = "" then
				exit function
			end if
		next

		'Find the best total score among the players.
		for i = 0 to UBound(users)

			'Get the player's total score.
			score = 0
			for week = SIDE_START_WEEK to finalWeek
				n = GetMarginScore(users(i), week)
				if IsNumeric(n) then
					score = score + n
				end if
			next

			'Get the number of correct picks the player had.
			correct = 0
			sql = "SELECT COUNT(*)AS Total FROM Schedule, SidePicks" _
			   & " WHERE Schedule.Week = SidePicks.Week" _
			   & " AND Schedule.Week >= " & SIDE_START_WEEK _
			   & " AND Username = '" & SqlEscape(users(i)) & "'" _
			   & " AND Pick = Result"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				correct = rs.Fields("Total").Value
			end if

			'If this is the first player, initialize the comparison values.
			if i = 0 then
				highScore   = score - 1
				mostCorrect = correct = 1
			end if

			'If this player has a higher score, make the player the winner.
			if score > highScore or (score = highScore and correct > mostCorrect) then
				redim list(0)
				list(0)     = users(i)
				highScore   = score
				mostCorrect = correct

			'Otherwise, if this player has the same score and the same number
			'of correct picks, add the player to the winners list.
			elseif score = highScore and correct = mostCorrect then
				redim preserve list(UBound(list) + 1)
				list(UBound(list)) = users(i)
			end if

		next

		if IsArray(list) then
			GetMarginWinnersList = list
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the week that the survivor pool was won. If that pool has not
	' been concluded yet, an empty string is returned.
	'--------------------------------------------------------------------------
	function GetSurvivorFinalWeek()

		dim curWeek, finalWeek
		dim sql, rs

		GetSurvivorFinalWeek = ""

		'Make sure we have the most current status.
		curWeek = GetCurrentWeek()
		sql = "SELECT COUNT(*) AS Total FROM SurvivorStatus" _
		   & " WHERE Week = " & curWeek
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then

			'If there is no status for the current week, set it.
			if rs.Fields("Total").Value = 0 then
				SetSurvivorStatus(curWeek)

			'Otherwise, check for any pending status and, if found, reset the
			'current week's status. (This must be done because in some cases, a
			'pending status may be finalized once all games for the week have
			'been locked.)
			else
				sql = "SELECT COUNT(*) AS Total FROM SurvivorStatus" _
				   & " WHERE Week = " & curWeek _
				   & " AND IsPending = 'TRUE'"
				set rs = DbConn.Execute(sql)
				if not (rs.EOF and rs.BOF) then
					if rs.Fields("Total").Value > 0 then
						SetSurvivorStatus(curWeek)
					end if
				end if
			end if
		end if

		'Check for a winner.
		sql = "SELECT Min(Week) AS FinalWeek FROM SurvivorStatus WHERE IsWinner = 'TRUE'"
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			finalWeek = rs.Fields("FinalWeek").Value
			if IsNumeric(finalWeek) then
				GetSurvivorFinalWeek = finalWeek
			end if
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns the current amount in the pot for the survivor pool.
	'--------------------------------------------------------------------------
	function GetSurvivorPotAmount()

		dim pot
		dim week
		dim rs, sql

		GetSurvivorPotAmount = 0

		'Get the base amount.
		GetSurvivorPotAmount =  GetSideEntryCount() * SIDE_ENTRY_FEE
		if ENABLE_MARGIN_POOL then
			GetSurvivorPotAmount = SIDE_SURVIVOR_SHARE * GetSurvivorPotAmount
		end if

		'Add on buyback fees.
		if SURVIVOR_BUYBACK_LIMIT > 0 then
			week = GetCurrentWeek()
			call SetSurvivorStatus(week)
			sql = "SELECT SUM(Buybacks) AS Total FROM SurvivorStatus" _
			   & " WHERE Week = " & week
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				GetSurvivorPotAmount = GetSurvivorPotAmount + (SURVIVOR_BUYBACK_FEE * rs.Fields("Total").Value)
			end if
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns a record set containing information about given user's status in
	' the survivor pool for the given week. If no data is found, an empty
	' record set is returned.
	'--------------------------------------------------------------------------
	function GetSurvivorStatus(username, week)

		dim sql

		'Search for the record..
		sql = "SELECT * FROM SurvivorStatus" _
		   & " WHERE Week = " & week _
		   & " AND Username = '" & SqlEscape(username) & "'"
		set GetSurvivorStatus = DbConn.Execute(sql)
		if not (GetSurvivorStatus.EOF and GetSurvivorStatus.BOF) then
			exit function
		else
			'No record found, try building the status information and search
			'again.
			call SetSurvivorStatus(week)
   			set GetSurvivorStatus = DbConn.Execute(sql)
		end if
        
	end function

	'--------------------------------------------------------------------------
	' Determines the winner of the survivor pool. An array of those player
	' names is returned. If the pool has not been concluded, or if no players
	' participated in it, an empty string is returned instead.
	'--------------------------------------------------------------------------
	function GetSurvivorWinnersList()

		dim sql, rs, week, list

		GetSurvivorWinnersList = ""

		'If the pool has not been won yet, exit.
		week = GetSurvivorFinalWeek()
		if not IsNumeric(week) then
			exit function
		end if

		'Check the status.
		sql = "SELECT Username FROM SurvivorStatus" _
		   & " WHERE Week = " & week _
		   & " AND IsWinner = 'TRUE'"
        
		set rs = DbConn.Execute(sql)
		do while not rs.EOF
			if not IsArray(list) then
				redim list(0)
			else
				redim preserve list(UBound(list) + 1)
			end if
			list(UBound(list)) = rs.Fields("Username").Value
			rs.MoveNext
		loop
		if IsArray(list) then
			GetSurvivorWinnersList = list
			exit function
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns true if the given user made a buyback for the specified week.
	'--------------------------------------------------------------------------
	function HasSurvivorBuyback(username, week)

		dim sql, rs

		HasSurvivorBuyback = false
		sql = "SELECT SurvivorBuyback FROM SidePicks" _
		   & " WHERE Username = '" & SqlEscape(username) & "'" _
		   & " AND Week = " & week
		set rs = DbConn.Execute(sql)
		if not (rs.EOF and rs.BOF) then
			HasSurvivorBuyback = rs.Fields("SurvivorBuyback").Value
		end if

	end function

	'--------------------------------------------------------------------------
	' Returns true if the given user may make a buyback for the given week.
	'--------------------------------------------------------------------------
	function IsSurvivorBuybackAllowed(username, week)

		dim rs
		dim maxWeek
		dim buybackCount

		'Assume a buyback is not permitted.
		IsSurvivorBuybackAllowed = false

		'If buybacks are not permitted, exit.
		if SURVIVOR_BUYBACK_LIMIT <= 0 then
			exit function
		end if

		'If the survivor pool is concluded, exit.
		maxWeek = GetSurvivorFinalWeek()
		if IsNumeric(maxWeek) then
			if maxWeek <= week then
				exit function
			end if
		end if

		'Determine the last week buybacks are allowed.
		maxWeek = GetWeekCount()
		if SURVIVOR_MAX_BUYBACK_WEEK > 0 and SURVIVOR_MAX_BUYBACK_WEEK < maxWeek then
			maxWeek = SURVIVOR_MAX_BUYBACK_WEEK
		end if
		if week <= SIDE_START_WEEK or week > maxWeek then
			exit function
		end if

		'Get the user's status for the given week.
		set rs = GetSurvivorStatus(username, week)

		'If the user is at the buyback limit, exit.
		buybackCount = rs.Fields("Buybacks").Value
		if buybackCount > SURVIVOR_BUYBACK_LIMIT then
			exit function
		elseif buybackCount = SURVIVOR_BUYBACK_LIMIT  and not HasSurvivorBuyback(username, week) then
			exit function
		end if

		'If the user was not eliminated the previous week, exit.
		set rs = GetSurvivorStatus(username, week - 1)
		if rs.Fields("WasAlive").Value ="FALSE" and rs.Fields("IsAlive").Value = "FALSE" then
			exit function
		end if

		'Otherwise, permit a buyback.
		IsSurvivorBuybackAllowed = true

	end function

	'--------------------------------------------------------------------------
	' Determines the outcome for each player in the survivor pool for the given
	' week and saves that information to the database.
	'--------------------------------------------------------------------------
	sub SetSurvivorStatus(week)

		dim sql, rs
		dim allLocked, anyPending, anyCorrect
		dim users, gamesRs
		dim completed, missed, revived, buybacks, wasAlive, isAlive, isPending, isWinner
		dim pick, result
		dim i, j

		'Exit if the week is invalid.
		if week < SIDE_START_WEEK then
			exit sub
		end if

		'Clear any existing status information.
		'ClearSurvivorStatus(week)
		sql = "DELETE FROM SurvivorStatus WHERE Week >= " & week
		DbConn.Execute(sql)

		'If we have data from the previous week and there were one or more
		'winners, just copy the data to this week (except for the the WasAlive
		'field which should be set to the same value as the IsAlive field of
		'the previous week) and exit.
		if week > SIDE_START_WEEK then
			sql = "SELECT * FROM SurvivorStatus" _
			   & " WHERE Week = " & week - 1 _
			   & " AND IsWinner = 'TRUE'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				sql = "SELECT * FROM SurvivorStatus WHERE Week = " & week - 1
				set rs = DbConn.Execute(sql)
				do while not rs.EOF
					sql = "INSERT INTO SurvivorStatus" _
					   & "(Week, Username, Completed, Missed, Revived, Buybacks, WasAlive, IsAlive, IsPending, IsWinner)" _
					   & "VALUES(" _
					   & week & ", " _
					   & "'" & SqlEscape(rs.Fields("Username").Value) & "', " _
					   & rs.Fields("Completed").Value & ", " _
					   & rs.Fields("Missed").Value & ", " _
					   & rs.Fields("Revived").Value & ", " _
					   & rs.Fields("Buybacks").Value & ",'" _
					   & rs.Fields("IsAlive").Value & "','" _
					   & rs.Fields("IsAlive").Value & "','" _
					   & rs.Fields("IsPending").Value & "','" _
					   & rs.Fields("IsWinner").Value & "')"
                   call DbConn.Execute(sql)
					rs.MoveNext
				loop
				exit sub
			end if
		end if

		'Get a list of users participating in the survivor pool.
		users = GetSidePlayersList()
		if not IsArray(users) then
			exit sub
		end if

		'Determine if all games for the week have been locked and initialize
		'the flags that indicates pending results.
		allLocked = IsWeekLocked(week)
		anyPending = false
		anyCorrect = false

		'Get all games for the week.
		set gamesRs = GetGamesRecordset(week)

		'For each user, determine the result of the week's pick.
		for i = 0 to UBound(users)

			'If this is the first week, initialize the results data. Otherwise,
			'load the previous week's data.
			if week = SIDE_START_WEEK then
				completed = 0
				missed    = 0
				revived   = 0
				buybacks  = 0
				wasAlive  = true
				isAlive   = true
				isPending = false
				isWinner  = false
			else
				sql = "SELECT * FROM SurvivorStatus" _
				   & " WHERE Week = " & week - 1 _
				   & " AND Username = '" & SqlEscape(users(i)) & "'"
				set rs = DbConn.Execute(sql)
				if not (rs.EOF and rs.BOF) then
					completed = rs.Fields("Completed").Value
					missed    = rs.Fields("Missed").Value
					revived   = rs.Fields("Revived").Value
					buybacks  = rs.Fields("Buybacks").Value
					wasAlive  = rs.Fields("WasAlive").Value
					isAlive   = rs.Fields("IsAlive"). Value
					isPending = rs.Fields("IsPending").Value
					isWinner  = rs.Fields("IsWinner").Value
				else
					'The previous week's results are not in the database so we
					'need to recalculate them and start processing over for
					'this week.
					call SetSurvivorStatus(week - 1)
					call SetSurvivorStatus(week)
					exit sub
				end if
			end if

			'If the user was eliminated last week, check for a buyback.
			if wasAlive and not isAlive then
				if HasSurvivorBuyback(users(i), week) then
					buybacks = buybacks + 1
					isAlive = true
				end if
			end if

			'Set the user's previous status.
			wasAlive = isAlive

			'If the user is alive, check this week's result.
			if isAlive then

				'Get the user's pick for the current week.
				pick = GetSidePick(users(i), week)

				'Handle a missing pick
				if pick = "" then

					'If all games are locked, count it as a miss. Otherwise,
					'mark the results as pending.
					if allLocked then
						completed = completed + 1
						missed = missed + 1
					else
						anyPending = true
					end if
				else

					'Find the game for that pick.
					gamesRs.MoveFirst
					do while not gamesRs.EOF
						if pick = gamesRs.Fields("VisitorID").Value or pick = gamesRs.Fields("HomeID").Value then

							'Check the result. If there is none, mark results
							'as pending. If a pick is correct, make a note of
							'it.
							result = gamesRs.Fields("Result").Value
							if not IsNull(result) then
								completed = completed + 1
								if pick <> result and not (result = TIE_STR and not SURVIVOR_STRIKE_ON_TIE) then
									missed = missed + 1
								else
									anyCorrect = true
								end if
							else
								anyPending = true
							end if
							exit do
						end if
						gamesRs.MoveNext
					loop
				end if

				'Determine if the user has been eliminated.
				if missed - (revived + buybacks) >= SURVIVOR_STRIKE_OUT then
					isAlive = false
				end if
			end if

			'Save the results for this user.
			sql = "INSERT INTO SurvivorStatus" _
			   & "(Week, Username, Completed, Missed, Revived, Buybacks, WasAlive, IsAlive, IsPending, IsWinner)" _
			   & "VALUES(" _
			   & week & ", " _
			   & "'" & SqlEscape(users(i)) & "', " _
			   & completed & ", " _
			   & missed & ", " _
			   & revived & ", " _
			   & buybacks & ", " _
			   & "'" & wasAlive & "', " _
			   & "'" & isAlive & "', " _
			   & "'" & isPending & "', " _
			   & "'" & isWinner & "')"
			call DbConn.Execute(sql)
		next
      
		'If all players have been eliminated, revive any who started the week
		'alive.
		if not anyPending then
			sql = "SELECT COUNT(*) AS Total From SurvivorStatus" _
			   & " WHERE Week = " & week _
			   & " AND IsAlive ='TRUE'"
		    set rs = DbConn.Execute(sql)
            if not (rs.EOF and rs.BOF) then
				if rs.Fields("Total").Value = 0 then
					sql = "UPDATE SurvivorStatus" _
					   & " SET IsAlive = 'TRUE'," _
					   & " Revived = Revived + 1," _
					   & " IsPending = 'FALSE'" _
					   & " WHERE Week = " & week _
					   & " AND WasAlive ='TRUE'"
					response.Write(sql)
                    call DbConn.Execute(sql)
				
                end if
			end if

		'Otherwise, if no player has made a correct pick yet, mark any
		'eliminations as pending.
		elseif not anyCorrect then
			sql = "UPDATE SurvivorStatus" _
			   & " SET IsPending = 1" _
			   & " WHERE Week = " & week _
			   & " AND WasAlive ='TRUE' AND IsAlive = 'FALSE'"
			call DbConn.Execute(sql)
		end if

		'Determine if we have a winner.
		if not anyPending then
			'Find out how many players survived.
			sql = "SELECT COUNT(*) AS Total From SurvivorStatus" _
			   & " WHERE Week = " & week _
			   & " AND IsAlive = 'TRUE'"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				if rs.Fields("Total").Value = 1 then
					'Only one player survived, make that player the winner.
					sql = "UPDATE SurvivorStatus" _
					   & " SET IsWinner = 'TRUE'" _
					   & " WHERE Week = " & week _
					   & " AND IsAlive = 'TRUE'"
					call DbConn.Execute(sql)
					exit sub
				elseif week = GetWeekCount() then

					'All weeks completed, make the surviving player(s) with the
					'fewest misses the winner(s).
					sql = "UPDATE SurvivorStatus" _
					   & " SET IsWinner = 'TRUE'" _
					   & " WHERE Week = " & week _
					   & " AND IsAlive = 'TRUE'" _
					   & " AND Missed = (" _
					   & "   SELECT MIN(Missed) FROM SurvivorStatus" _
					   & "   WHERE Week = " & week _
					   & "   AND IsAlive = 'TRUE')"
					call DbConn.Execute(sql)
				end if
			end if
		end if
  
	end sub

	'--------------------------------------------------------------------------
	' Removes unused buyback in the survivor pool and if possible, applies
	' them as needed in later weeks.
	'
	' Note: This should be called whenever ClearSurvivorStatus() is called.
	'--------------------------------------------------------------------------
	sub ShiftSurvivorBuybacks()

		dim curWeek
		dim users
		dim i, maxBuybacks, usedBuybacks
		dim week
		dim sql, rs

		'Get the most current results.
		curWeek = GetCurrentWeek()
		call SetSurvivorStatus(curWeek)

		'Process each user in the survivor pool.
		users = GetSidePlayersList()
		if not isArray(users) then
			exit sub
		end if
		for i = 0 to UBound(users) - 1

			'Find the number of buybacks the user currently has.
			maxBuybacks  = 0
			usedBuybacks = 0
			sql = "SELECT * FROM SidePicks" _
			   & " WHERE Username = '" & SqlEscape(users(i)) & "'" _
			   & " AND SurvivorBuyback ='TRUE'" _
			   & " AND Week <= " & curWeek _
			   & " ORDER BY Week"
			set rs = DbConn.Execute(sql)
			if not (rs.EOF and rs.BOF) then
				do while not rs.EOF
					if rs.Fields("SurvivorBuyback").Value = "TRUE" then
						maxBuybacks = maxBuyBacks + 1
					end if
					rs.MoveNext
				loop
				if maxBuybacks > SURVIVOR_BUYBACK_LIMIT then
					maxBuybacks = SURVIVOR_BUYBACK_LIMIT
				end if

				'Remove any buybacks that were not needed.
				rs.MoveFirst
				do while not rs.EOF
					if rs.Fields("SurvivorBuyback").Value ="TRUE" then
						week = rs.Fields("Week").Value
						if UsedSurvivorBuyback(users(i), week) then
							usedBuybacks = usedBuybacks + 1
						else
							sql = "UPDATE SidePicks" _
							  & " SET SurvivorBuyback = 'FALSE'" _
							  & " WHERE Username = '" & SqlEscape(users(i)) & "'" _
							  & " AND Week = " & week
							DbConn.Execute(sql)
						end if
					end if
					rs.MoveNext
				loop

				'Now, check if the user can apply any left over buybacks in
				'later weeks.
				for week = SIDE_START_WEEK + 1 to curWeek
					if usedBuybacks >= maxBuybacks then
						exit for
					end if
					if GetSidePick(users(i), week) <> "" then
						set rs = GetSurvivorStatus(users(i), week)
						if rs.Fields("IsAlive").Value = "FALSE" then
							if IsSurvivorBuybackAllowed(users(i), week) then
								if not HasSurvivorBuyback(users(i), week) then
									sql = "UPDATE SidePicks" _
									  & " SET SurvivorBuyback = 'TRUE'" _
									  & " WHERE Username = '" & SqlEscape(users(i)) & "'" _
									  & " AND Week = " & week
									DbConn.Execute(sql)
									call SetSurvivorStatus(week)
									usedBuybacks = usedBuybacks + 1
								end if
							end if
						end if
					end if
					rs.MoveNext
				next
			end if
		next

	end sub

	'--------------------------------------------------------------------------
	' Returns true if the given user used a buyback for the specified week.
	'--------------------------------------------------------------------------
	function UsedSurvivorBuyback(username, week)

		dim n1, n2
		dim rs

		UsedSurvivorBuyback = false

		if week <= SIDE_START_WEEK then
			exit function
		end if

		set rs = GetSurvivorStatus(username, week)
		n1 = rs.Fields("Buybacks").Value
		set rs = GetSurvivorStatus(username, week - 1)
		n2 = rs.Fields("Buybacks").Value

		if n1 <> n2 then
			UsedSurvivorBuyback = true
		end if

	end function
 %>
